/*
** $Id: conf-internal.h,v 1.4 2003/01/10 00:58:40 pief Exp $
**
** MetalServe common stuff
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** conf.c-internal header file for configuration parser functions
**
** $Log: conf-internal.h,v $
** Revision 1.4  2003/01/10 00:58:40  pief
** Introduced section support in the config file parser, which includes
** heavy changes to the interface and the operation of argument parser
** functions.
** Replaced malloc() calls with calloc() -- there's no reason not to use it.
** Added an argument parser functions for ranges of integer values.
** Extended copyright note to the new year.
**
** Revision 1.3  2002/10/25 15:59:37  pief
** Replaced names of per-file #define's to avoid possible namespace clashes
**
** Revision 1.2  2002/09/24 22:05:13  pief
** For some reason I checked in ancient versions, so here are the current
** ones
**
*/

#ifndef _METALSERVE_CONF_INTERNAL_H
#define _METALSERVE_CONF_INTERNAL_H

#include <stdio.h>

#include "list.h"

/*
** Maximum support length of lines in the configuration file
*/
#define MAX_CFG_LINELEN 1024

/*
** The CFGFILE structure keeps everything related to a particular configuration file.
*/
struct _cfgfile
{
	FILE		*fp;				/* The stream used to read the config file */

	unsigned int	flags;				/* Flags associated with this config file */

	unsigned int	cfgerrno;			/* Configuration error code (as defined below) */
	char		*cfgerrstr;			/* Associated error message */

	char		linebuf[MAX_CFG_LINELEN],	/* Contents of last line read */
			*linep,				/* Pointer into linebuf */
			*dstr,				/* Pointer to directive part */
			*argstr;			/* Pointer to argument part */
	unsigned int	lineno;				/* Current line number */
	
	CFGDIRECTIVE	*d;				/* Current directive */

	unsigned short	current_section;		/* Current section ID (eg. GLOBAL_SECTION) */
	void		*current_structp;		/* Current section's structure pointer (NULL when in global section) */

	LIST		*sectstack;			/* Stack for processing of subsections (empty when in global section) */
};

/*
** The CFGSECTSTACKNODE structure is used to remember the properties
** of parent sections during the processing of subsections.
*/
typedef struct _cfgsectstacknode
{
	NODE		n;				/* Pointer to next and previous node in list */

	char		*drtvname;			/* Name of directive that opened this section */
	unsigned int	drtvlineno;			/* Line number in which this section was opened */

	unsigned short	parent_section;			/* Parent section's ID */
	void		*parent_structp;		/* Parent section's structp */
} CFGSECTSTACKNODE;

/*
** Defines for the internal state machine of the cfgread() function
** as regards opening sections
*/
typedef enum _sectopen_state_t
{
	MUST_NOT_OPEN, CAN_OPEN, MUST_OPEN, DOES_OPEN
} SECTOPEN_STATE_T;

void cfgerr(CFGFILE *cfgfile, unsigned int cfgerrno);

#endif
