/*
** $Id: list.h,v 1.5 2003/01/10 00:51:48 pief Exp $
**
** MetalServe common stuff
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** Header file for list functions
**
** $Log: list.h,v $
** Revision 1.5  2003/01/10 00:51:48  pief
** Extended copyright note to the new year
** Correct indentation to use Tabs instead of Spaces
** Added #defines for minimum and maximum values for the priority in a
** PRIONODE
**
** Revision 1.4  2002/10/25 15:59:37  pief
** Replaced names of per-file #define's to avoid possible namespace clashes
**
** Revision 1.3  2002/10/08 00:22:48  pief
** Added missing includes to make the common stuff compile on FreeBSD
**
** Revision 1.2  2002/09/24 22:05:13  pief
** For some reason I checked in ancient versions, so here are the current
** ones
**
*/

#ifndef _METALSERVE_LIST_H
#define _METALSERVE_LIST_H

#include <sys/types.h>
#include <netinet/in.h>

/*
** Structures used for double-linked lists
*/

/* Simple node structure */
typedef struct _node
{
	struct _node	*next,		/* Next node in list */
			*prev;		/* Previous node in list */
} NODE;

/* Extended node structure for priority-sorted lists */
typedef struct _prionode
{
	struct _node	*next,		/* Next node in list */
			*prev;		/* Previous node in list */

	int		priority;	/* Priority field */
} PRIONODE;

/* Limits for the "priority" field above. Use in your own programs, the list.c
   routines don't care about these! */
#define MINPRI -19
#define MAXPRI  20

/* General list header */
typedef struct _list
{
	int		count;		/* Number of nodes in list */

	struct _node	*first,		/* First node in list */
			*last;		/* Last node in list */
} LIST;

/* Some typedefs to ease passing a callback function as argument */
typedef int (* traverse_node_func)(NODE *n);
typedef void (* free_node_func)(NODE *n);

/*
** Public function prototypes
*/

/* List functions */
LIST *allocate_list(void);
void free_list(LIST *list, free_node_func func);
NODE *traverse_list(LIST *list, traverse_node_func func);
void insert_node(LIST *list, NODE *node, NODE *new_node);
#define append_node(list, new_node) insert_node(list, list->last, new_node)
void insert_prionode(LIST *list, PRIONODE *new_prionode);
void remove_node(LIST *list, NODE *node);

#endif
