/*
** $Id: log.h,v 1.3 2003/01/10 00:50:38 pief Exp $
**
** MetalServe common stuff
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** Header file for logging functions
**
** $Log: log.h,v $
** Revision 1.3  2003/01/10 00:50:38  pief
** Extended copyright note to the new year
** Renamed DEFAULT_LOGLEVEL to DEF_LOGLEVEL for shorter code lines
**
** Revision 1.2  2002/11/24 10:52:30  pief
** Have log.h define a DEFAULT_LOGLEVEL
**
** Revision 1.1  2002/10/25 16:09:17  pief
** Added logging routines
**
**
*/

#ifndef _METALSERVE_LOG_H
#define _METALSERVE_LOG_H

#include <syslog.h>

#define LOG_TIMESTAMP_FORMAT	"[%b %e %T]"	/* strftime() format, yields "[MMM DD hh:mm:ss]" */
#define LOG_TIMESTAMPBUF_LENGTH	50		/* buffer for the timestamp and the program name */
#define LOG_OUTPUTBUF_LENGTH	2048		/* size of output buffer */

#define DEF_LOGLEVEL		LOG_NOTICE	/* defines a default for use by programs that call logopen() */

/*
** Public function prototypes
*/
int getloglevelval(const char *name);
const char *getloglevelname(const int loglevel);
BOOL logopen(const char *logfile, const int loglevel);
void logclose(void);
void do_logmsg(const char *file, const char *func, const int line, const int severity,
               const BOOL errmsg, const char *fmt, ...);

/* Always use this macro instead of do_logmsg() to prevent the preprocessor
   from using log.c as __FILE__ etc. (see log.c) */
#define logmsg(sev, fmt...) do_logmsg(__FILE__, __func__, __LINE__, sev, 0, fmt)
/* Use the following macro for error messages which should also be written to stderr
   if a tty is connected */
#define logmsg_stderr(sev, fmt...) do_logmsg(__FILE__, __func__, __LINE__, sev, 1, fmt)

#endif
