/*
** $Id: main.h,v 1.6 2003/01/10 00:59:34 pief Exp $
**
** makelist - MetalServe file list generator
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** Header file with declarations and definitions
**
** $Log: main.h,v $
** Revision 1.6  2003/01/10 00:59:34  pief
** Extended copyright note to the new year
**
** Revision 1.5  2002/11/24 11:03:14  pief
** Made INTxy_[LE|BE] macros use CHAR_BIT defines for bit shifting
**
** Revision 1.4  2002/10/19 22:35:25  pief
** Renamed makelist.[ch] to main.[ch]
**
** Revision 1.3  2002/10/01 07:54:54  pief
** Moved rotation delay into #define in makelist.h
**
** Revision 1.2  2002/09/30 21:42:02  pief
** Removed video definitions which are to be placed in forthcoming video
** modules.
** Added define for makelist's output buffer size
**
** Revision 1.1  2002/09/24 21:46:03  pief
** Added first version of makelist main files
**
*/

#ifndef _MAKELIST_MAIN_H
#define _MAKELIST_MAIN_H

#include <limits.h>

/* Macro to generate a 16-bits integer from a four byte, little-endian (lo-hi, LSB) byte sequence inside a char array */
#define INT16_LE(array, offset) (array[offset+1] << CHAR_BIT) + (array[offset])

/* Macro to generate a 32-bits integer from a four byte, big-endian (hi-lo, MSB) byte sequence inside a char array */
#define INT32_BE(array, offset) (array[offset] << 3 * CHAR_BIT) + (array[offset+1] << 2 * CHAR_BIT) + \
                                (array[offset+2] << CHAR_BIT) + (array[offset+3])

/* Macro to generate a 32-bits integer from a four byte, little-endian (lo-hi, LSB) byte sequence inside a char array */
#define INT32_LE(array, offset) (array[offset+3] << 3 * CHAR_BIT) + (array[offset+2] << 2 * CHAR_BIT) + \
                                (array[offset+1] << CHAR_BIT) + (array[offset])

/*
** Configuration directives supported within makelist
*/
extern char *_logfile_name;
extern char *_serverroot;
extern char *_compress_command;
extern BOOL *_keep_uncompressed_list;
extern char **_include_extensions;
extern char **_nickname;

/*
** Internal node structure used to represent scanned directories and files
*/
typedef struct _scannode
{
	struct _node *next,			/* Next node in list */
	             *prev;			/* Previous node in list */

	char *name,				/* Directory or file name */
	     *fullname;				/* Complete file name with full path */

	LIST *contents;				/* Contents if this is a subdirectory (size = 0) */
	int size;				/* File size in bytes */

	int infotype;				/* Type of additional information present */
	void *info;				/* Filetype-specific information */
} SCANNODE;

/* Macro to test whether a SCANNODE represents a directory */
#define IS_DIR(node) (node->size == 0)

/* Definitions for the infotype field */
enum
{
	INFOTYPE_NONE, INFOTYPE_MPEGAUDIO, INFOTYPE_WAVEAUDIO
};

/*
** Miscellaneous stuff
*/

/* Size of output buffer */
#define BUF_SIZE 10240

/* Delay between each rotation in progress display in microseconds */
#define ROTATION_DELAY 250000

#endif
