/*
** $Id: mpegaudio.h,v 1.3 2003/01/10 00:59:34 pief Exp $
**
** makelist - MetalServe file list generator
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** MPEG audio info functions header file
**
** $Log: mpegaudio.h,v $
** Revision 1.3  2003/01/10 00:59:34  pief
** Extended copyright note to the new year
**
** Revision 1.2  2002/10/25 16:03:33  pief
** Replaced names of per-file #define's to avoid possible namespace clashes
**
** Revision 1.1  2002/09/24 21:46:23  pief
** Added first version of MPEG Audio-related functions
**
*/

#ifndef _METALSERVE_MPEGAUDIO_H
#define _METALSERVE_MPEGAUDIO_H

typedef struct _mpegaudioinfo
{
	int mpeg_version,			/* MPEG audio version */
	    mpeg_layer;				/* MPEG audio layer */

	int length,				/* Length in seconds */
	    bitrate,				/* Bitrate in kbps (0=VBR) */
	    samplingfreq,			/* Sampling frequency in Hz */
	    channelmode,			/* Channelmode */
	    padded;				/* Padded flag */

	int mpegsize,				/* Total size of actual MPEG data (without ID3 tags etc.) */
	    frames;				/* Number of MPEG frames */

	char encoder_string[50];		/* Additional information about encoder & settings used */

	char id3v1_songtitle[31],		/* Song title from ID3v1 tag */
	     id3v1_artist[31],			/* Artist from ID3v1 tag */
	     id3v1_album[31];			/* Album from ID3v1 tag */
	int id3v1_track,			/* Track from ID3v1.1 tag */
	    id3v1_year;				/* Year from ID3v1 tag */
	int id3v1_genre;			/* Genre from ID3v1 tag */
} MPEGAUDIOINFO;

/* Length of MPEG audio frame header */
#define MPEG_AUDIO_FRAME_HEADERLEN 4

/* Do not change the order of the following enums -- they corrospend with MPEG bit definitions */
enum
{
	MPEG_AUDIO_V2, MPEG_AUDIO_V1
};

enum
{
	MPEG_AUDIO_LAYER3, MPEG_AUDIO_LAYER2, MPEG_AUDIO_LAYER1
};

enum
{
	CHANNELMODE_STEREO, CHANNELMODE_JSTEREO, CHANNELMODE_DUALMONO, CHANNELMODE_MONO
};

/* Multiplicators in bytes_per_frame formulas in makelist.c */
#define L1_BPF_MULT			(12000.0 * 4.0)
#define L2_L3_BPF_MULT			144000

/* Multiplicators in time_per_frame formulas in makelist.c */
#define L1_TPF_MULT			384
#define L2_L3_TPF_MULT			1152

/* Xing VBR header definitions */
#define XING_ID				"Xing"
#define XING_STEREO_OFFSET		32
#define XING_MONO_OFFSET		17
#define XING_FLAGS_OFFSET		7
#define XING_FRAMES_OFFSET		8
#define XING_FRAMES_FLAG		1

/* Lame header definitions */
#define LAME_CBR_ID			"Info"
#define LAME_ID				"LAME"
#define LAME_OFFSET			120
#define LAME_VER_OFFSET			124
#define LAME_QUALLEVEL_OFFSET		119

/* Frauenhofer VBR header definitions */
#define FRAUENHOFER_ID			"VBRI"
#define FRAUENHOFER_OFFSET		32
#define FRAUENHOFER_FRAMES_OFFSET	46
#define FRAUENHOFER_QUALLEVEL_OFFSET	41

/* Definitions for the headerless VBR detection routine */
#define VBR_DETECT_FRAMES		19
#define VBR_AVGCALC_FRAMES		499

/* ID3v1 tag definitions */
#define ID3V1_LEN			128
#define ID3V1_ID			"TAG"
#define ID3V1_SONGTITLE_OFFSET		3
#define ID3V1_SONGTITLE_LENGTH		30
#define ID3V1_ARTIST_OFFSET		33
#define ID3V1_ARTIST_LENGTH		30
#define ID3V1_ALBUM_OFFSET		63
#define ID3V1_ALBUM_LENGTH		30
#define ID3V1_YEAR_OFFSET		93
#define ID3V1_COMMENT_OFFSET		97
#define ID3V1_GENRE_OFFSET		127
#define ID3V1_TRACK_OFFSET		126 /* ID3 v1.1 */
#define ID3V1_GENRE_MAXID		147 /* highest assigned genre ID */

/*
** Function prototypes
*/
BOOL examine_mpegaudio_file(FILE *file, char *filename, SCANNODE *node);
void get_mpegaudio_info_string(MPEGAUDIOINFO *info, char *buf);

#endif
