/*
** $Id: main.h,v 1.1 2003/01/10 01:02:38 pief Exp $
**
** metalserved - IRC DCC file server
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** Header file with declarations and definitions
**
** $Log: main.h,v $
** Revision 1.1  2003/01/10 01:02:38  pief
** Added first version of metalserved, which yet does nothing but full
** configuration file processing, including subsections.
**
**
*/

#ifndef _METALSERVED_MAIN_H
#define _METALSERVED_MAIN_H

#include "../common/conf.h"

/*
** Global configuration variables
*/
extern char *_logfile_name;
extern char *_serverroot;
extern char **_nickname;
extern char *_realname;
extern char *_email_address;

extern int _server_connect_timeout;
extern int _server_timeout;
extern int _part_timeout;
extern BOOL _part_finishdccs;
extern BOOL _client_lookups;
extern char **_ignored_users;
extern int _message_linedelay;
extern int _max_find_results;
extern int _dcc_minport,
           _dcc_maxport;
extern int _dcc_timeout;
extern int _dcc_maxxfers,
           _dcc_maxqueue;
extern int _dcc_maxuserxfers,
           _dcc_maxuserqueue;
extern int _dcc_maxlistxfers,
           _dcc_maxlistqueue;
extern int _dcc_mincps;
extern int _dcc_largefiles_threshold,
           _dcc_largefiles_mincps;

/*
** Length restrictions for argparser_string configuration variables
*/
#define MAX_REALNAME_LEN		70	/* Maximum length for "realname" directive */
#define MAX_EMAILADDR_LEN		70	/* Maximum length for "email_address" directive */
#define MAX_SERVER_LEN			100	/* Maximum length for "server" directive */
#define MAX_PASSWORD_LEN		40	/* Maximum length for "password" directive */
#define MAX_CHANNEL_LEN			30	/* Maximum length for "channel" directive */

/*
** Defaults for most global configuration variables
*/
#define DEF_SERVER_CONNECT_TIMEOUT	30	/* Timeout on connect() to IRC servers */
#define DEF_SERVER_TIMEOUT		300	/* Timeout on stalling IRC server connections */
#define DEF_PART_TIMEOUT		120	/* Timeout for PARTing users */
#define DEF_PART_FINISHDCCS		TRUE	/* Finish DCC SENDs to PARTing users */
#define DEF_CLIENT_LOOKUPS		FALSE	/* Resolve IP address of clients? */
#define DEF_MSG_LINEDELAY		100	/* Delay in milliseconds after newline */
#define DEF_MAX_FIND_RESULTS		10	/* Max. # results in response to @find/@locator */
#define DEF_DCC_MINPORT			1024	/* Lower bound for port range for DCC SENDs */
#define DEF_DCC_MAXPORT			1100	/* Upper bound for port range for DCC SENDs */
#define DEF_DCC_TIMEOUT			180	/* Timeout on DCC SENDs */
#define DEF_DCC_MAXXFERS		2	/* Max. # of concurrent xfers */
#define DEF_DCC_MAXQUEUE		20	/* Queue length */
#define DEF_DCC_MAXUSERXFERS		1	/* Max. # of concurrent xfers per user */
#define DEF_DCC_MAXUSERQUEUE		3	/* Max. queued requests per user */
#define DEF_DCC_MAXLISTXFERS		3	/* Max. # of concurrent xfers of filelist */
#define DEF_DCC_MAXLISTQUEUE		30	/* Max. # of queued filelist requests */
#define DEF_DCC_MINCPS			0	/* Minimum required CPS */
#define DEF_DCC_LARGEFILES_THRESHOLD	10240	/* Threshold for large files */
#define DEF_DCC_LARGEFILES_MINCPS	0	/* Minimum required CPS for large files */

/*
** Restrictions for argparser_int configuration variables
*/
#define MAX_ADVERTISE_DELAY		86400	/* advertise at least once a day */
#define MAX_TIMEOUT			3600	/* 1 hour as upper limit for all timeouts */
#define MAX_MSG_LINEDELAY		1000	/* 1 second as upper limit */
#define MAX_MAX_FIND_RESULTS		100	/* Max. 100 results */
#define MIN_DCC_PORT			1024	/* Lowest port number usable without running as "root" user */
#define MAX_DCC_PORT			65535	/* Highest possible port number */
#define MAX_XFERS			100	/* Highest imaginable, reasonable value */
#define MAX_QUEUE			500	/* Highest imaginable, reasonable value */
#define MAX_CPS				9999999	/* Never underestimate the speed of light */
#define MAX_LARGEFILES_THRESHOLD	1048576	/* You wouldn't want to further raise the threshold */

/*
** Section IDs for configuration directives that (can/must) open own sections (2^x with x > 0 since
** 2^0 is reserved for the GLOBAL_SECTION)
*/
#define NETWORK_SECTION			2
#define SERVER_SECTION			4
#define CHANNEL_SECTION			8

/*
** Definitions for the "state" field of the successively defined TCPCONN structure.
*/
typedef enum _tcpconn_state_t
{
	TCPCONN_UNRESOLVED,			/* Disconnected, hostname unsolved */
	TCPCONN_DISCONNECTED,			/* Disconnected, hostname resolved */
	TCPCONN_CONNECTING,			/* Connection in progress */
	TCPCONN_CONNECTED,			/* Connection succeeded */
	TCPCONN_LISTEN				/* Connection is being listened on */
} TCPCONN_STATE_T;

/*
** A TCPCONN object is a low-level structure used to keep track of the actual TCP/IP connections
** and the associated parameters such as the target IP address.
*/
typedef struct _tcpconn
{
	struct sockaddr_in	addr;		/* IP address and port number */
	char			*name;		/* Associated hostname obtained from DNS */

	int			socket;		/* Socket file descripted used for communication */

	TCPCONN_STATE_T		state;		/* State of connection */
} TCPCONN;

/*
** Prototypes for argument parser functions
*/
APFUNC(argparser_network);
APFUNC(argparser_server);
APFUNC(argparser_channel);

#endif
