/*
** $Id: servers.c,v 1.1 2003/01/10 01:02:38 pief Exp $
**
** metalserved - IRC DCC file server
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** IRC server handling code
**
** $Log: servers.c,v $
** Revision 1.1  2003/01/10 01:02:38  pief
** Added first version of metalserved, which yet does nothing but full
** configuration file processing, including subsections.
**
**
*/

#include "../common/base.h"
#include "../common/list.h"

#include "main.h"
#include "servers.h"

/*
** network = allocate_network()
**
** Allocates and initializes a new NETWORK structure. Returns NULL if not enough
** memory was available.
*/
NETWORK *allocate_network(void)
{
	NETWORK *network;

	network = (NETWORK *)calloc(sizeof(NETWORK), 1);
	if (network)
	{        
		network->channels = allocate_list();
		if (network->channels)
		{        
			network->servers = allocate_list();
			if (network->servers)
			{
				network->ignored_users = allocate_list();
				if (network->ignored_users)
				{
					network->all_disabled = FALSE;

					return (network);
				}

				free(network->servers);
			}

			free(network->channels);
		}

		free(network);
        }

	return NULL;
}

/*
** free_network(network)
**
** Frees all memory associated with the specified NETWORK structure. This includes
** freeing the memory associated with the appropriate CHANNEL and SERVER structures.
*/
void free_network(NETWORK *network)
{
	if (network->name)
		free(network->name);
	if (network->channels)
		free_list(network->channels, (free_node_func)free);
	if (network->servers)
		free_list(network->servers, (free_node_func)free);
	if (network->ignored_users)
		free_list(network->ignored_users, (free_node_func)free);

	free(network);
}

/*
** server = allocate_server()
**
** Allocates and initializes a new SERVER structure. Returns NULL if not enough
** memory was available.
*/
SERVER *allocate_server(void)
{
	SERVER *server;
	extern SERVER _defserver;

	server = (SERVER *)calloc(sizeof(SERVER), 1);
	if (server)
	{
		server->state      = SERVER_DISCONNECTED;
		server->conn.state = TCPCONN_UNRESOLVED;
		server->disabled   = FALSE;

		server->n.priority = _defserver.n.priority;

		server->minport    = _defserver.minport;
		server->maxport    = _defserver.maxport;

		if (_defserver.password)
		{
			server->password   = (char *)strdup(_defserver.password);
			if (!server->password)
			{
				free(server);
				return NULL;
			}
		}

		return server;
	}

	return NULL;
}

/*
** free_server(server)
**
** Frees all memory associated with the specified SERVER structure.
*/
void free_server(SERVER *server)
{
	if (server->conn.name)
		free(server->conn.name);

	free(server);
}

/*
** channel = allocate_channel()
**
** Allocates and initializes a new CHANNEL structure. Returns NULL if not enough
** memory was available.
*/
CHANNEL *allocate_channel(void)
{
	CHANNEL *channel;
	extern CHANNEL _defchannel;

	channel = (CHANNEL *)calloc(sizeof(CHANNEL), 1);
	if (channel)
	{
		channel->joined = FALSE;

		channel->advertise_delay = _defchannel.advertise_delay;
		channel->find_trigger    = _defchannel.find_trigger;

		return channel;
	}

	return NULL;
}

/*
** free_channel(channel)
**
** Frees all memory associated with the specified CHANNEL structure.
*/
void free_channel(CHANNEL *channel)
{
	if (channel->name)
		free(channel->name);

	free(channel);
}
