/*
** $Id: servers.h,v 1.1 2003/01/10 01:02:38 pief Exp $
**
** metalserved - IRC DCC file server
** Copyright (c) 2002-2003 by Pieter Hollants <contact@metalserve.org>
**
** This program is licensed under the GNU General Public License, version 2,
** as published by the Free Software Foundation and available in the file
** COPYING and the Internet location http://www.gnu.org/licenses/gpl.html.
**
** This program is distributed in the hope that it will be useful, but
** WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
** or FITNESS FOR A PARTICULAR PURPOSE.
**
** IRC server handling code header file
**
** $Log: servers.h,v $
** Revision 1.1  2003/01/10 01:02:38  pief
** Added first version of metalserved, which yet does nothing but full
** configuration file processing, including subsections.
**
**
*/

#ifndef _METALSERVED_SERVERS_H
#define _METALSERVED_SERVERS_H

#include <netinet/in.h>

/*
** Definitions for the internal state machine of connect_servers() that keeps track
** of the state of connections to IRC servers
*/
typedef enum _server_state_t
{
	SERVER_DISCONNECTED,			/* No server connection yet */
	SERVER_SENDPASSWORD,			/* Send PASSWORD message */
	SERVER_SENDNICK,			/* Send NICK message */
	SERVER_SENDUSER,			/* Send USER message */
	SERVER_REGISTERED,			/* Connection registered, there are channels to be joined */
	SERVER_JOINED				/* Connection registered, channels joined */
} SERVER_STATE_T;

/*
** A SERVER structure represents a configured or dynamically added server to which
** MetalServe might try to connect to.
*/
typedef struct _server
{
	PRIONODE	n;			/* Next/previous server in list */

	TCPCONN		conn;			/* TCPCONN object */

	int		minport,		/* Port range server is listening on. These variables */
			maxport;		/* are written by the config parser and are converted
						   on-the-fly to an in_port_t. The actual port used is
						   stored in conn->addr.sin_port. */

	time_t		connecttime;		/* Time of last/for next connect (0 = now) */
	BOOL		disabled;		/* Has the server been disabled? */

	char		*password;		/* Password required to connect to server */

	SERVER_STATE_T	state;			/* IRC protocol state */	
} SERVER;

/*
** Compiled-in defaults for the fields of the SERVER structure
*/
#define DEF_SERVER_MINPORT 6667
#define DEF_SERVER_MAXPORT 6667

/*
** A CHANNEL structure represents a configured or dynamically added channel that is
** to be joined by MetalServe.
*/
typedef struct _channel
{
	PRIONODE	n;			/* Next/previous channel in list */
        
	char		*name;			/* Channel name */
        
        int		advertise_delay;	/* Delay in seconds for advertising */
	BOOL		find_trigger;		/* React to @find / @locator? */

	BOOL		joined;			/* Is this channel currently joined? */
} CHANNEL;

/*
** Compiled-in defaults for the fields of the CHANNEL structure
*/
#define DEF_CHANNEL_ADVERTISEDELAY 900
#define DEF_CHANNEL_FINDTRIGGER TRUE

/*
** A NETWORK structure represents a configured or dynamically added IRC network to
** which MetalServe will try to connect to.
*/
typedef struct _network
{
	NODE		n;			/* Next/previous network in list */

	char		*name;			/* IRC network name */
	
	LIST		*servers;		/* List of servers in this network */
	SERVER		*current_server;	/* Currently selected server */
	BOOL		all_disabled;		/* If TRUE, all servers have been disabled */

	LIST		*channels;		/* List of channels in this network */

	LIST		*ignored_users;		/* List of ignored users in this IRC network */
} NETWORK;

/*
** A NICKNAME structure is one of many entries in a list of configured nicknames.
** Only one nickname can be active at any time, indicated by the "used" field.
*/
typedef struct _nickname
{
	NODE		n;			/* Next/previous network in list */

	char		*name;			/* Nickname */

	int		priority,		/* Priority of nickname */
			used;			/* Nickname in use? */	
} NICKNAME;

/*
** An IRCMSG structure represents the splitted-up parts of a message received
** from or sent to an IRC server.
*/
typedef struct _ircmsg
{
	char		*prefix,		/* Optional prefix */
			*command,		/* Command */
			*params;		/* Optional parameters */
} IRCMSG;

/*
** Function prototypes
*/

#endif
